@@ -23,6 +23,7 @@ import re
 
 from docutils import nodes
 from docutils.parsers import rst
+import six
 from six.moves import configparser
 
 KEY_PATTERN = re.compile("[^a-zA-Z0-9_]")
@@ -104,7 +105,13 @@ class Matrix(object):
                     "one of (%s)" % (option, status, section, ", ".join(
                         Implementation.STATUS_ALL)))
 
-            impl = Implementation(status)
+            option_notes = ''.join([DRIVER_NOTES_PREFIX,
+                                    option[len(DRIVER_PREFIX):]])
+            notes = None
+            if cfg.has_option(section, option_notes):
+                notes = cfg.get(section, option_notes)
+
+            impl = Implementation(status=status, notes=notes)
             feature.implementations[option] = impl
 
             return feature
@@ -160,8 +167,9 @@ class Implementation(object):
     STATUS_ALL = [STATUS_COMPLETE, STATUS_MISSING,
                   STATUS_PARTIAL, STATUS_UNKNOWN]
 
-    def __init__(self, status=STATUS_MISSING):
+    def __init__(self, status=STATUS_MISSING, notes=None):
         self.status = status
+        self.notes = notes
 
 
 STATUS_SYMBOLS = {
@@ -205,8 +213,14 @@ class Directive(rst.Directive):
         env = self.state.document.settings.env
         fname = self.arguments[0]
         rel_fpath, fpath = env.relfn2path(fname)
+
+        # Handle deprecation of readfp in py3 for read_file that was not
+        # available in py2.
+        if six.PY2:
+            cfg.read_file = cfg.readfp
+
         with open(fpath) as fp:
-            cfg.readfp(fp)
+            cfg.read_file(fp)
 
         # This ensures that the docs are rebuilt whenever the
         # .ini file changes
@@ -375,6 +389,9 @@ class Directive(rst.Directive):
                                   ids=[key_id]),
                 ]
 
+                if impl.notes is not None:
+                    subitem.append(self._create_notes_paragraph(impl.notes))
+
                 impls.append(subitem)
 
             para_divers.append(impls)
